#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdarg.h>
#include "jlib.h"
#define STACKSIZ 32

typedef struct {
  int    str_stack_num;
  char  *str_stackbuf[STACKSIZ];
  char **str_stack;
} jconv_stack;

char *push_jconv_stack(jconv_stack *st, int code, char *str)
{
    char *jstr = NULL;

    if (st->str_stack_num < STACKSIZ) {
	switch (code) {
	    case  JIS: jstr = toStringJIS(str); break;
	    case  EUC: jstr = toStringEUC(str); break;
            case SJIS: jstr = toStringSJIS(str); break;
            default:   jstr = toStringAuto(str); break;
	}
	if (jstr) {
	    *(st->str_stack++) = jstr;
	    st->str_stack_num++;
	}
    }

    return jstr;
}

int pop_jconv_stack(jconv_stack *st)
{
    if (st->str_stack_num > 0) {
        free(*(--st->str_stack));
	st->str_stack_num--;
	return 1;
    }
    return 0;
}

jconv_stack *create_jconv_stack()
{
    jconv_stack* st = (jconv_stack *)malloc(sizeof(jconv_stack));

    st->str_stack = st->str_stackbuf;
    st->str_stack_num = 0;
    return st;
}

void delete_jconv_stack(jconv_stack *st)
{
    if (st) {
        while (pop_jconv_stack(st));
	free(st);
    }
}

#define jconv_va_end(ap) \
{ \
    va_end(ap); \
}


#define jconv_va_start(code, st, ap, fmt) \
{ \
    char *fmt_ptr = fmt; \
    unsigned char c; \
    int i; \
    double d; \
    char *s; \
    va_start(ap, fmt); \
    while ((c = *fmt_ptr++)) { \
      if (c == '%') { \
	int flag = 0; \
	while ((c = *fmt_ptr++)) { \
	    if (flag == 0) { \
	      if ( c == '%') \
		break; \
	      if (c == '-' ||  c == '+' ||  c == ' ') \
		continue; \
	    } \
	    flag = 1; \
	    if (isdigit(c) || c == '.') \
	      continue; \
	    if (isalpha(c)) \
	      break; \
	} \
	switch (c) \
	  { \
	    case 'd': case 'i': case 'o': case 'x': case 'X': \
	        i = va_arg(ap, int); break; \
	    case 'e': case 'E': case 'f': case 'g': \
	        d = va_arg(ap, double); break; \
	    case 'c': \
	        c = va_arg(ap, char); break; \
	    case 's': \
	        { \
		    char **str_ptr = (char**)ap, *jstr; \
		    jstr = push_jconv_stack(st, code, *str_ptr); \
		    if (jstr) *str_ptr = jstr; \
		} \
		s = va_arg(ap, char*); break; \
	  } \
      } \
    } \
    va_end(ap); \
} \
va_start(ap,fmt);


static void jconv_va_arg_convert(int code, int n, va_list ap, char *fmt)
{
    char *fmt_ptr = fmt;
    unsigned char c;
    int i;
    double d;
    char *s;

    while ((c = *fmt_ptr++) && n >0) {
      if (c == '%') {
	int flag = 0;
	while ((c = *fmt_ptr++)) {
	    if (flag == 0) {
	      if ( c == '%')
		break;
	      if (c == '-' ||  c == '+' ||  c == ' ')
		continue;
	    }
	    flag = 1;
	    if (isdigit(c) || c == '.')
	      continue;
	    if (isalpha(c))
	      break;
	}
	switch (c)
	  {
	    case 'd': case 'i': case 'o': case 'x': case 'X':
	        i = *va_arg(ap, int*); n--; break;
	    case 'e': case 'E': case 'f': case 'g':
	        d = *va_arg(ap, double*); n--; break;
	    case 'c':
	        c = *va_arg(ap, char*); n--; break;
	    case 's':
		{
		    char **str_ptr = (char**)ap, *jstr;
		    switch(code) {
			case  JIS: jstr = toStringJIS(*str_ptr); break;
			case  EUC: jstr = toStringEUC(*str_ptr); break;
			case SJIS: jstr = toStringSJIS(*str_ptr); break;
			default:   jstr = toStringAuto(*str_ptr); break;
		    }
		    if (jstr) {
		        strcpy(*str_ptr, jstr);
			free(jstr);
		    }
		s = va_arg(ap, char*); n--; break;
		}
	  }
      }
    }
}

int jconv_printf(int code, char *fmt, ...)
{
  va_list ap;
  int n;
  char *jfmt;
  jconv_stack *st = create_jconv_stack();

  jfmt = push_jconv_stack(st, code, fmt);

  jconv_va_start(code, st, ap, fmt);
  n = vprintf (jfmt, ap);
  jconv_va_end(ap);

  delete_jconv_stack(st);

  return n;
}

int jconv_fprintf(int code, FILE *fp, char *fmt, ...)
{
  va_list ap;
  int n;
  char *jfmt;
  jconv_stack *st = create_jconv_stack();

  jfmt = push_jconv_stack(st, code, fmt);

  jconv_va_start(code, st, ap, fmt);
  n = vfprintf (fp, jfmt, ap);
  jconv_va_end(ap);

  delete_jconv_stack(st);

  return n;
}

int jconv_snprintf(int code, char *buf, int len, char *fmt, ...)
{
  va_list ap;
  int n;
  char *jfmt;
  jconv_stack *st = create_jconv_stack();

  jfmt = push_jconv_stack(st, code, fmt);

  jconv_va_start(code, st, ap, fmt);
  n = vsnprintf (buf, len, jfmt, ap);
  jconv_va_end(ap);

  delete_jconv_stack(st);

  return n;
}

int jconv_sprintf(int code, char *buf, char *fmt, ...)
{
  va_list ap;
  int n;
  char *jfmt;
  jconv_stack *st = create_jconv_stack();

  jfmt = push_jconv_stack(st, code, fmt);

  jconv_va_start(code, st, ap, fmt);
  n = vsprintf (buf, jfmt, ap);
  jconv_va_end(ap);

  delete_jconv_stack(st);

  return n;
}

int jconv_scanf(int code, char *fmt, ...)
{
  va_list ap;
  int n;

  va_start(ap, fmt);
  n = vscanf (fmt, ap);
  va_end(ap);

  va_start(ap, fmt);
  jconv_va_arg_convert(code, n, ap, fmt);
  va_end(ap);

  return n;
}

int jconv_fscanf(int code, FILE *fp, char *fmt, ...)
{
  va_list ap;
  int n;

  va_start(ap, fmt);
  n = vfscanf (fp, fmt, ap);
  jconv_va_arg_convert(code, n, ap, fmt);
  va_end(ap);

  return n;
}

int jconv_sscanf(int code, char *buf, char *fmt, ...)
{
  va_list ap;
  int n;

  va_start(ap, fmt);
  n = vsscanf (buf, fmt, ap);
  jconv_va_arg_convert(code, n, ap, fmt);
  va_end(ap);

  return n;
}

