#define USE_PLUGIN

#include <stdio.h>
#include <stdlib.h>
#include "../../src/common/xchat.h"
#include "../../src/common/plugin.h"
#include "jlib.h"

extern	struct module *module_find (char *name);

int language_cmd (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
int	lang_send (int **skip, int sok, char *buf, int len, void *a, char c);
int	lang_recv (int **skip, int **len, int sok, char *lbuf, int size, char c);

char	*name = "lang-jp";
char	*desc = "Japanese display plugin";

struct	commands	language_cmds[] = {
	{"LANGUAGE",	language_cmd,	0, 0, "/LANGUAGE"},
	{0, 0, 0, 0, 0}
};

struct	xp_signal	send_sig, recv_sig;
int	(*send_next)	(void *, void *, void *, void *, void *, char);
int	(*recv_next)	(void *, void *, void *, void *, void *, char);

struct	module_cmd_set	language_cmd_set;

int	module_init (int ver, struct module *mod, struct session *sess)
{
	/* This check *MUST* be done first */
	if (ver != MODULE_IFACE_VER)
		return 1;
	
	if (module_find (name) != NULL) {
		/* We are already loaded */
		PrintText(sess, "Language Module already loaded\n");
		return 1;
	}
	PrintText(sess, "Loaded module Language: Japanese (ܸ)\n");
	mod->name = name;
	mod->desc = desc;
		
	send_sig.signal = XP_IF_SEND;
	send_sig.callback = XP_CALLBACK(lang_send);
	send_sig.naddr = &send_next;
	send_sig.mod = mod;
	
	recv_sig.signal = XP_IF_RECV;
	recv_sig.callback = XP_CALLBACK(lang_recv);
	recv_sig.naddr = &recv_next;
	recv_sig.mod = mod;
	
	hook_signal(&recv_sig);
	hook_signal(&send_sig);

	language_cmd_set.mod = mod;
	language_cmd_set.cmds = language_cmds;
	
	module_add_cmds (&language_cmd_set);
	
	return 0;
}

void module_cleanup (struct module *mod, struct session *sess)
{
	PrintText(sess, "Language Japanese module unloading\n");
}

int	lang_send (int **skip, int sok, char *buf, int len, void *a, char c)
{
	char	*text;
	
	if (*skip == 0) {
		//printf("lang_send[%d:%d]%d: %s\n", len, strlen(buf), detectKanjiCode(buf), buf);
		if ((text = toStringJIS(buf))) {
			buf = text;
			len = strlen(text);
		}
		//printf("         [%d]%d> %s\n", len, detectKanjiCode(buf), buf);
		send (sok, buf, len, 0);
		if (text) free(text);
		*skip = 1;
	}
	XP_CALLNEXT(send_next, skip, sok, buf, len, a, c);
}

int	lang_recv (int **skip, int **len, int sok, char *lbuf, int size, char c)
{
	char	*text;
	int		llen;

	if (*skip == 0) {
		llen = recv (sok, lbuf, size, 0);
		if (llen >= 1) {
			*(lbuf+llen) = 0;
			//printf("lang_recv[%d:%d]%d: %s\n", llen, strlen(lbuf), detectKanjiCode(lbuf), lbuf);
			if ((text = toStringAuto(lbuf))) {
				strcpy(lbuf, text);
				llen = strlen(text);
				free(text);
			}
			//printf("         [%d]%d> %s\n", llen, detectKanjiCode(lbuf), lbuf);
		}
		*len = llen;
		*skip = 1;
	}
	XP_CALLNEXT(recv_next, skip, len, sok, lbuf, size, c);
}

int	language_cmd (struct session *sess, char *tbuf, char *word[], char *word_eol[])
{
	PrintText(sess, "Language Japanese Module version 0.1\n");
	return 0;
}
